@@ -92,13 +92,27 @@ var formatter = function(value, indent, parents) {
       if (parents.indexOf(value) >= 0) return '[Circular]';
       parents.push(value);
 
-      if (value instanceof Array) {
+      if (value instanceof Array && Object.keys(value).length === value.length) {
         return formatObject(value, indent, parents, '[]', function(x, f) {
           return f(value[x]);
         });
       } else {
         return formatObject(value, indent, parents, '{}', function(x, f) {
-          return f(x) + ': ' + f(value[x]);
+          var child;
+          if (value.__lookupGetter__(x)) {
+            if (value.__lookupSetter__(x)) {
+              child = "[Getter/Setter]";
+            } else {
+              child = "[Getter]";
+            }
+          } else {
+            if (value.__lookupSetter__(x)) {
+              child = "[Setter]";
+            } else {
+              child = f(value[x]);
+            }
+          }
+          return f(x) + ': ' + child;
         });
       }
       return buffer;
@@ -113,6 +127,7 @@ var formatter = function(value, indent, parents) {
 var formatObject = function(obj, indent, parents, parenthesis, entryFormatter) {
   var buffer = parenthesis[0];
   var values = [];
+  var x;
 
   var localFormatter = function(value) {
     return formatter(value, indent + ' ', parents);
