@@ -18,7 +18,7 @@ var server = tcp.createServer(function (socket) {
 
   socket.addListener("close", function (had_error) {
     //puts("server had_error: " + JSON.stringify(had_error));
-    assertFalse(had_error);
+    assert.equal(false, had_error);
   });
 });
 server.listen(port);
@@ -34,13 +34,13 @@ client.addListener("connect", function () {
 client.addListener("receive", function (chunk) {
   client_recv_count += 1;
   puts("client_recv_count " + client_recv_count);
-  assertEquals("hello\r\n", chunk);
+  assert.equal("hello\r\n", chunk);
   client.close();
 });
 
 client.addListener("close", function (had_error) {
   puts("disconnect");
-  assertFalse(had_error);
+  assert.equal(false, had_error);
   if (disconnect_count++ < N)
     client.connect(port); // reconnect
   else
@@ -48,6 +48,6 @@ client.addListener("close", function (had_error) {
 });
 
 process.addListener("exit", function () {
-  assertEquals(N+1, disconnect_count);
-  assertEquals(N+1, client_recv_count);
+  assert.equal(N+1, disconnect_count);
+  assert.equal(N+1, client_recv_count);
 });
