@@ -36,3 +36,7 @@ def detect(conf):
 
   found = os.path.exists(join(prefix, "bin/node"))
   conf.check_message('node prefix', '', found, prefix)
+
+  ## On Mac OSX we need to use mac bundles
+  if Options.platform == 'darwin': conf.check_tool('osx')
+
